# -*- coding: utf-8 -*-

import ipaddress
import re
from lib import status

class TargetHost:
    """バージョンチェックの対象となるホスト"""
    
    def __init__(self, strline):
        """コンストラクタ。strlineは http(s)?://<password>@<IPAddress>:<port>/ 形式のstr"""
        (https, ipAddress, port, password) = self._getAddrFromLine(strline)
        
        # IPアドレスが指定の形式で無かったらException
        self._https = https
        self._ipAddress = ipaddress.ip_address(ipAddress)
        if (1 <= port) and (port <= 65535):
            self._port = port
        else:
            raise ValueError()
        self._password = password
        
        self.hostStatus = status.HostStatus.NOTSTART
        self._version = None
        self.errorstr = ''

    def getURLBase(self):
        """ベースURLを文字列で返す"""
        if self._https:
            scheme = 'https'
        else:
            scheme = 'http'
        
        return '{0}://{1}:{2}'.format(scheme, self._ipAddress, self._port)

    def setVersion(self, version):
        self._version = version

    def getVersion(self):
        return self._version

    def getHost(self):
        return self._ipAddress
    
    def getPassword(self):
        return self._password

    def __str__(self):
        return "TargetHost: ipaddr={0}, port={1}, password={2}, https={3}, version={4}"\
            .format(self._ipAddress, self._port, self._password, self._https, self._version)
    
    def _getAddrFromLine(self, line):
        """1行の文字列から、プロトコル、IPアドレス、ポート、パスワードを取得する"""
        https = False
        port = 80
        password = 'system'
        
        prog = re.compile('(https?)://(([^@]+)@)?([^:/]+)(:([0-9]+))?/?')
        
        if not isinstance(line, str):
            raise TypeError()
        
        m = prog.fullmatch(line)
        if m is None:
            raise ValueError()
        
        t = m.groups()
        
        if t[0] == 'https':
            https = True
            port = 443
        if t[2] is not None:
            password = t[2]
        ipAddress = t[3]
        if t[5] is not None:
            port = int(t[5])
        
        return (https, ipAddress, port, password)

    @staticmethod
    def getTargetHosts(filename):
        """IP:PORT:PASSWORDの組が書いてある複数行のテキストファイルから、TargetHostのリストとエラーになった行を返す"""
        targetHosts = []
        errorLines = []
        
        for line in open(filename):
            line = line.strip()
            if len(line) == 0 or line.startswith('#'):
                # 空行やコメント行なので飛ばす
                continue
            
            try:
                targetHost = TargetHost(line)
                targetHosts.append(targetHost)
            except Exception as ex:
                print(ex)
                errorLines.append(line)
        
        return (targetHosts, errorLines)

def testTargetHosts():
    (targetHosts, errorLines) = TargetHost.getTargetHosts("list.txt")
    for targetHost in targetHosts:
        print(targetHost)
    
    print("---")
    
    for errorLine in errorLines:
        print(errorLine)

if __name__ == "__main__":
    testTargetHosts()
